@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2019 Osimis S.A., Belgium
+ * Copyright (C) 2017-2020 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -324,16 +324,25 @@ namespace Orthanc
     
   Json::Value DicomWebJsonVisitor::FormatDouble(double value)
   {
-    long long a = boost::math::llround<double>(value);
-
-    double d = fabs(value - static_cast<double>(a));
-
-    if (d <= std::numeric_limits<double>::epsilon() * 100.0)
+    try
     {
-      return FormatInteger(a);
+      long long a = boost::math::llround<double>(value);
+
+      double d = fabs(value - static_cast<double>(a));
+
+      if (d <= std::numeric_limits<double>::epsilon() * 100.0)
+      {
+        return FormatInteger(a);
+      }
+      else
+      {
+        return Json::Value(value);
+      }
     }
-    else
+    catch (boost::math::rounding_error&)
     {
+      // Can occur if "long long" is too small to receive this value
+      // (e.g. infinity)
       return Json::Value(value);
     }
   }
@@ -512,15 +521,21 @@ namespace Orthanc
       Json::Value& node = CreateNode(parentTags, parentIndexes, tag);
       node[KEY_VR] = EnumerationToString(vr);
 
+#if 0
+      /**
+       * TODO - The JSON file has an UTF-8 encoding, thus DCMTK
+       * replaces the specific character set with "ISO_IR 192"
+       * (UNICODE UTF-8). On Google Cloud Healthcare, however, the
+       * source encoding is reported, which seems more logical. We
+       * thus choose the Google convention. Enabling this block will
+       * mimic the DCMTK behavior.
+       **/
       if (tag == DICOM_TAG_SPECIFIC_CHARACTER_SET)
       {
-        // TODO - The JSON file has an UTF-8 encoding, thus DCMTK
-        // replaces the specific character set with "ISO_IR 192"
-        // (UNICODE UTF-8). It is unclear whether the source
-        // character set should be kept: We thus mimic DCMTK.
         node[KEY_VALUE].append("ISO_IR 192");
       }
       else
+#endif
       {
         std::string truncated;
         
@@ -533,12 +548,21 @@ namespace Orthanc
         {
           truncated = value;
         }
-        
+
         if (!truncated.empty())
         {
           std::vector<std::string> tokens;
           Toolbox::TokenizeString(tokens, truncated, '\\');
 
+          if (tag == DICOM_TAG_SPECIFIC_CHARACTER_SET &&
+              tokens.size() > 1 &&
+              tokens[0].empty())
+          {
+            // Specific character set with code extension: Remove the
+            // first element from the vector of encodings
+            tokens.erase(tokens.begin());
+          }
+
           node[KEY_VALUE] = Json::arrayValue;
           for (size_t i = 0; i < tokens.size(); i++)
           {
@@ -575,30 +599,45 @@ namespace Orthanc
                 }
                   
                 case ValueRepresentation_IntegerString:
-                  if (tokens[i].empty())
+                {
+                  /**
+                   * The calls to "StripSpaces()" below fix the
+                   * issue reported by Rana Asim Wajid on 2019-06-05
+                   * ("Error Exception while invoking plugin service
+                   * 32: Bad file format"):
+                   * https://groups.google.com/d/msg/orthanc-users/T32FovWPcCE/-hKFbfRJBgAJ
+                   **/
+
+                  std::string t = Orthanc::Toolbox::StripSpaces(tokens[i]);
+                  if (t.empty())
                   {
                     node[KEY_VALUE].append(Json::nullValue);
                   }
                   else
                   {
-                    int64_t value = boost::lexical_cast<int64_t>(tokens[i]);
+                    int64_t value = boost::lexical_cast<int64_t>(t);
                     node[KEY_VALUE].append(FormatInteger(value));
                   }
-                  
+                 
                   break;
+                }
               
                 case ValueRepresentation_DecimalString:
-                  if (tokens[i].empty())
+                {
+                  std::string t = Orthanc::Toolbox::StripSpaces(tokens[i]);
+                  if (t.empty())
                   {
                     node[KEY_VALUE].append(Json::nullValue);
                   }
                   else
                   {
-                    double value = boost::lexical_cast<double>(tokens[i]);
+                    double value = boost::lexical_cast<double>(t);
                     node[KEY_VALUE].append(FormatDouble(value));
                   }
+
                   break;
-              
+                }
+
                 default:
                   if (tokens[i].empty())
                   {
@@ -614,7 +653,15 @@ namespace Orthanc
             }
             catch (boost::bad_lexical_cast&)
             {
-              throw OrthancException(ErrorCode_BadFileFormat);
+              std::string tmp;
+              if (value.size() < 64 &&
+                  Toolbox::IsAsciiString(value))
+              {
+                tmp = ": " + value;
+              }
+              
+              LOG(WARNING) << "Ignoring DICOM tag (" << tag.Format()
+                           << ") with invalid content for VR " << EnumerationToString(vr) << tmp;
             }
           }
         }
